<template>
  <div class="mod-config">
    <basic-container>
      <el-row>
        <el-col :span="13">
          <ttable
            :table="table"
            @currentChange="handleCurrentChange"
            :uploadInfo="uploadInfo"
            :prelang="prelang"
            :options="options"
            :ajaxFun="ajaxFun"
            ref="customerOrderTable"
          >
            <template #toolbar></template>
          </ttable>
        </el-col>
        <el-col :span="11" style="padding-left: 10px">
          <ttable
            :table="joinTable"
            @handleSelectionChange="joinHandleSelectionChange"
            :uploadInfo="joinUploadInfo"
            :prelang="joinPrelang"
            :options="joinOptions"
            :ajaxFun="joinAjaxFun"
            :paramObj="paramObj"
            ref="joinStockTable"
          >
            <template #toolbar></template>
          </ttable>
        </el-col>
      </el-row>

      <!-- 预留库存零件 -->
      <!--<stockDialog
        :currshowlist.sync="relateVisible"
        @handleSelectionChange="saveJoinData"
        :paramSelArr="paramSelArr"
        :paramSelCol="paramSelCol"
        :multiSelect="stockMultiSelect"
        :showIsUsed="false"
        :partNo="partNo"
      />-->
      <!-- 修改预留库存数量 -->
      <joinStockOrderDialog v-if="addOrUpdateVisible" ref="addOrUpdate" />
      <!-- 预留 -->
      <reservedCustomForm
        :currshowlist.sync="relateVisible"
        :currentCustomerOrder="currentCustomerOrder"
        @handleReserved="handleReserved"
      />
    </basic-container>
  </div>
</template>

<script>
import { mapGetters } from 'vuex'
import ttable from '@/views/common/ztt-table.vue'
import stockDialog from '@/views/common/stock'
import joinStockOrderDialog from './join-stock-order'
import reservedCustomForm from './reserved-custom-form'
import { fetchListCustomerOrder } from '@/api/plan/customerorder'
import {
  fetchList,
  fetchAll,
  addReserve,
  cancelReserve,
  changeState,
  updateOrder
} from '@/api/warehouse/joinstockorder'

export default {
  data() {
    return {
      ajaxFun: fetchListCustomerOrder,
      currentCustomerOrder: null,
      lastCustomerOrderRow: null,
      uploadInfo: {
        // 是否展示上传EXCEL以及对应的url
        isShow: false,
        url: ''
      },
      prelang: 'operation',
      options: {
        height: 300, // 默认高度-为了表头固定
        stripe: true, // 是否为斑马纹 table
        highlightCurrentRow: false, // 是否要高亮当前行
        border: true, // 是否有纵向边框
        lazy: false, // 是否需要懒加载
        fit: true, // 列的宽度是否自撑开
        multiSelect: false, //
        seqNo: true,
        isRefresh: true, // 是否显示刷新按钮
        isSearch: false, // 高级查询按钮
        cancelRunCreated: false,
        defaultOrderBy: { column: 'wantedDeliveryDate', direction: 'desc' }
      },
      table: {
        total: 0,
        currentPage: 1,
        pageSize: 20,
        data: [],
        // 标题
        column: [
          {
            minWidth: '100',
            prop: 'customerOrderNo',
            label: '客户订单号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'customerName',
            label: '客户名称',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'coLineNo',
            label: '行号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'coState',
            label: '客户订单状态',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            formatter: this.getCustomerCoState,
            optList: () => {
              return this.coStateList
            }
          },
          {
            minWidth: '120',
            width: '100px',
            prop: 'partNo',
            label: '零件号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            width: '100px',
            prop: 'partName',
            label: '零件名称',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'buyQtyDue',
            label: '销售数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'reservedQuantityRealtime',
            label: '已预留数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'deliveryQuantity',
            label: '已发货数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'number'
          },
          {
            minWidth: '140',
            width: '120',
            prop: 'wantedDeliveryDate',
            label: '期望交货时间',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'datetimerange',
            formatter: this.formatDate
          },
          {
            minWidth: '200',
            width: '120',
            prop: 'plannedDeliveryDate',
            label: '计划交货时间',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'datetimerange',
            formatter: this.formatDate
          }
        ],
        toolbar: [],
        operator: null,
        operatorConfig: {
          fixed: 'right',
          label: '操作',
          width: 200,
          minWidth: 100
        }
      },
      coStateList: [
        {
          value: '01plan',
          label: '待计划'
        },
        {
          value: '03planed',
          label: '已计划'
        },
        {
          value: '02planing',
          label: '计划中'
        },
        {
          value: '04complete',
          label: '已完成'
        }
      ],
      partNo: '',

      paramObj: { customerOrderId: 0 },
      joinAjaxFun: fetchList,
      joinMultipleSelection: [],
      joinUploadInfo: {
        // 是否展示上传EXCEL以及对应的url
        isShow: false,
        url: ''
      },
      joinPrelang: 'operation',
      joinOptions: {
        height: 300, // 默认高度-为了表头固定
        stripe: true, // 是否为斑马纹 table
        highlightCurrentRow: false, // 是否要高亮当前行
        border: true, // 是否有纵向边框
        lazy: false, // 是否需要懒加载
        fit: true, // 列的宽度是否自撑开
        multiSelect: true, //
        seqNo: true,
        isRefresh: true, // 是否显示刷新按钮
        isShowHide: true, // 是否显示显影按钮
        isSearch: false, // 高级查询按钮
        defaultOrderBy: { column: 'id', direction: 'asc' }
      },
      joinTable: {
        total: 0,
        currentPage: 1,
        pageSize: 20,
        data: [],
        // 标题
        column: [
          {
            minWidth: '100',
            prop: 'status',
            label: '状态',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            formatter: this.formatStatus,
            optList: () => {
              return this.statusOptions
            }
          },
          {
            minWidth: '110',
            prop: 'partNo',
            label: '零件编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '110',
            prop: 'partName',
            label: '零件名称',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '100',
            prop: 'warehouseName',
            label: '仓库',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '100',
            prop: 'locationNo',
            label: '库位号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '100',
            prop: 'partBatchNo',
            label: 'SN号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '100',
            prop: 'reelNumber',
            label: '载具编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '110',
            prop: 'reservedQuantity',
            label: '预留数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'number'
          },
          {
            minWidth: '110',
            prop: 'deliveryQuantity',
            label: '发货数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'number'
          },
          {
            minWidth: '100',
            prop: 'partUnit',
            label: '单位',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          }
        ],
        toolbar: [
          {
            text: '预留',
            type: 'primary',
            fun: this.reservedHandle,
            disabled: false
          },
          {
            text: '取消预留',
            type: 'primary',
            fun: this.cancelReservedHandle,
            disabled: false
          }
        ],
        operator: null,
        operatorConfig: {
          fixed: 'right',
          label: '操作',
          width: 60,
          minWidth: 60
        }
      },
      addOrUpdateVisible: false,
      statusOptions: [
        {
          value: '02confirmed',
          label: '已确认',
          colorLabel: '已确认'
        }
      ],
      paramSelArr: [],
      paramSelCol: 'id',
      stockMultiSelect: false,
      relateVisible: false
    }
  },
  components: {
    ttable,
    stockDialog,
    joinStockOrderDialog,
    reservedCustomForm
  },
  computed: {
    ...mapGetters(['permissions'])
  },
  watch: {},
  created() {
    const customNo = localStorage.getItem('masterschedule_customno')
    if (customNo != null && customNo !== 'null') {
      this.options.cancelRunCreated = true
    }
  },
  mounted() {
    const customNo = localStorage.getItem('masterschedule_customno')
    const partNo = localStorage.getItem('masterschedule_partno')
    const qtyRequired = localStorage.getItem('masterschedule_qtyrequired')
    if (customNo != null && customNo !== 'null') {
      const queryParam = []
      queryParam.push(
        Object.assign({
          prop: 'customerOrderNo',
          searchInfoType: 'text',
          propVal: customNo
        })
      )
      queryParam.push(
        Object.assign({
          prop: 'partNo',
          searchInfoType: 'text',
          propVal: partNo
        })
      )
      queryParam.push(
        Object.assign({
          prop: 'buyQtyDue',
          searchInfoType: 'text',
          propVal: qtyRequired
        })
      )

      this.$refs.customerOrderTable.setQueryParam(queryParam)
      this.getData()
      localStorage.removeItem('masterschedule_customno')
      localStorage.removeItem('masterschedule_partno')
      localStorage.removeItem('masterschedule_qtyrequired')
    }
  },
  methods: {
    getData() {
      return this.$refs.customerOrderTable.getDataList()
    },
    formatDate(row, column, cellValue) {
      if (cellValue) {
        const dutyDateList = /\d{4}-\d{1,2}-\d{1,2}/g.exec(cellValue)
        if (dutyDateList && dutyDateList.length > 0) {
          return dutyDateList[0]
        }
      }
      return ''
    },
    getCustomerCoState(row, column, cellValue) {
      this.coStateList.forEach((obj) => {
        if (obj.value === cellValue) {
          cellValue = obj.label
        }
      })
      return cellValue
    },
    handleCurrentChange(row) {
      // this.joinTable.toolbar.forEach((item) => {
      //   item.disabled = true
      // })
      if (row != null) {
        this.lastCustomerOrderRow = row
      }
      if (!row) {
        this.currentCustomerOrder = null
        this.paramObj = { customerOrderId: 0 }
        this.partNo = ''
      } else if (
        !this.currentCustomerOrder ||
        this.currentCustomerOrder.id !== row.id
      ) {
        this.currentCustomerOrder = row
        this.paramObj = { customerOrderId: row.id }
        this.partNo = row.partNo
        // if (row.coState === '03planed' || row.coState === '02planing' ) {
        //   this.joinTable.toolbar.forEach((item) => {
        //     item.disabled = false
        //   })
        // }
      }
      this.$nextTick(() => {
        this.getJoinData()
      })
    },

    getJoinData() {
      this.$refs.joinStockTable.getDataList()
    },
    joinHandleSelectionChange(val) {
      this.joinMultipleSelection = val
    },
    formatStatus(row, column, cellValue) {
      this.statusOptions.forEach((obj) => {
        if (obj.value === cellValue) {
          cellValue = obj.colorLabel
        }
      })
      return cellValue
    },
    // 修改预留数量
    addOrUpdateHandle(row) {
      this.addOrUpdateVisible = true
      this.$nextTick(() => {
        this.$refs.addOrUpdate.init(row)
      })
    },
    // 打开预留库存选择页面
    reservedHandle() {
      if (this.currentCustomerOrder) {
        this.relateVisible = true
      } else {
        this.$message.warning('请先选择一条客户订单')
      }
    },
    // 打开预留库存选择页面
    /* reservedHandle() {
      if (this.currentCustomerOrder) {
        // 筛选出当前已经预留的库存
        this.paramSelArr = []
        fetchAll(this.paramObj).then((response) => {
          if (response.data.data) {
            response.data.data.forEach((e) => {
              this.paramSelArr.push(e.stockId)
            })
          }
          this.relateVisible = true
          this.stockMultiSelect = true
        })
      } else {
        this.$message.warning('请先选择一条客户订单')
      }
    }, */
    // 保存预留库存
    saveJoinData(stockList, isUsed) {
      if (this.currentCustomerOrder && stockList && stockList.length > 0) {
        const joinStockOrderList = []
        stockList.forEach((item) => {
          // 可用库存大于0才能预留
          if (item.availableStockQuantity > 0) {
            joinStockOrderList.push(
              Object.assign({
                customerOrderId: this.currentCustomerOrder.id,
                stockId: item.id,
                reservedQuantity: item.availableStockQuantity,
                partNo: item.partNo,
                partName: item.partName,
                warehouseNo: item.warehouseNo,
                warehouseName: item.warehouseName,
                locationNo: item.locationNo,
                locationName: item.locationName,
                partUnit: item.unit,
                systemNo: item.systemNo,
                partBatchNo: item.partBatchNo,
                status: '01unconfirmed',
                type: '客户订单预留'
              })
            )
          }
        })
        if (joinStockOrderList && joinStockOrderList.length > 0) {
          addReserve(joinStockOrderList)
            .then((response) => {
              const resCode = response.data.code
              if (resCode === 0) {
                this.$refs.joinStockTable.getDataList()
                this.$message.success('预留成功')
              } else {
                this.$message.error('预留失败')
              }
            })
            .catch(() => {
              console.log('零件预留时，发生错误')
            })
        }
      }
    },
    // 取消预留
    cancelReservedHandle() {
      if (this.joinMultipleSelection && this.joinMultipleSelection.length > 0) {
        const joinStockOrderList = []
        let joinMultiple
        for (let i = 0; i < this.joinMultipleSelection.length; i++) {
          joinMultiple = this.joinMultipleSelection[i]
          joinStockOrderList.push({
            id: joinMultiple.id,
            customerOrderId: joinMultiple.customerOrderId,
            stockId: joinMultiple.stockId,
            reservedQuantity: joinMultiple.reservedQuantity,
            partNo: joinMultiple.partNo,
            partName: joinMultiple.partName,
            warehouseNo: joinMultiple.warehouseNo,
            warehouseName: joinMultiple.warehouseName,
            locationNo: joinMultiple.locationNo,
            locationName: joinMultiple.locationName,
            partUnit: joinMultiple.partUnit,
            systemNo: joinMultiple.systemNo,
            partBatchNo: joinMultiple.partBatchNo,
            status: '02confirmed',
            type: '客户订单预留'
          })
        }

        cancelReserve(joinStockOrderList)
          .then((response) => {
            const resCode = response.data.code
            if (resCode === 0) {
              this.handleReserved()
              this.$message.success('取消预留成功')
            } else {
              this.$message.error('取消预留失败')
            }
          })
          .catch(() => {
            console.log('取消预留时，发生错误')
          })
      } else {
        this.$message.warning('请选择需要取消预留的库存')
      }
    },
    // 确认
    confirmHandle() {
      if (this.joinMultipleSelection && this.joinMultipleSelection.length > 0) {
        for (let i = 0; i < this.joinMultipleSelection.length; i++) {
          if (this.joinMultipleSelection[i].status === '02confirmed') {
            this.$message.warning('请选择待确认的预留库存')
            return
          }
        }
        changeState(this.joinMultipleSelection, 'CONFIRM').then((response) => {
          this.$refs.joinStockTable.getDataList()
          this.$message.success('确认成功')
          this.refreshCustomerOrder()
        })
      } else {
        this.$message.warning('请选择需要确认的库存')
      }
    },
    // 取消确认
    cancelConfirmHandle() {
      if (this.joinMultipleSelection && this.joinMultipleSelection.length > 0) {
        for (let i = 0; i < this.joinMultipleSelection.length; i++) {
          if (this.joinMultipleSelection[i].status === '01unconfirmed') {
            this.$message.warning('请选择已确认的预留库存')
            return
          }
        }
        changeState(this.joinMultipleSelection, 'CANCEL').then((response) => {
          this.$refs.joinStockTable.getDataList()
          this.$message.success('取消确认成功')
          this.refreshCustomerOrder()
        })
      } else {
        this.$message.warning('请选择需要取消确认的库存')
      }
    },
    // 刷新客户订单的已预留数量
    refreshCustomerOrder() {
      if (this.currentCustomerOrder && this.currentCustomerOrder.id) {
        updateOrder(this.currentCustomerOrder.id).then((response) => {
          this.$set(
            this.currentCustomerOrder,
            'reservedQuantity',
            response.data.data
          )
        })
      }
    },
    // 预留后，刷新左、右列表
    handleReserved() {
      this.getData().then((message) => {
        // 获取客户预留数据，并将最近选中的点击选中
        this.$refs.customerOrderTable.clickTable(this.lastCustomerOrderRow)
      })
    }
  }
}
</script>
